import { Container } from 'inversify';
import { SyncBailHook, SyncHook, SyncWaterfallHook } from 'tapable';
import Clock from '../../utils/clock';
import { ISceneConfig } from '../config/IConfigService';
import { IMapService } from '../map/IMapService';
import { IBlendOptions, IModel, IModelInitializationOptions } from '../renderer/IModel';
import { IMultiPassRenderer, IPass, IPostProcessingPass } from '../renderer/IMultiPassRenderer';
import { IRendererService } from '../renderer/IRendererService';
import { IUniform } from '../renderer/IUniform';
import { ISource, ISourceCFG } from '../source/ISourceService';
import { IAnimateOption, IEncodeFeature, IScale, IScaleOptions, IStyleAttributeService, StyleAttrField, StyleAttributeOption, Triangulation } from './IStyleAttributeService';
export declare enum BlendType {
    normal = "normal",
    additive = "additive",
    subtractive = "subtractive",
    min = "min",
    max = "max"
}
export interface IBlendTypes {
    [key: string]: Partial<IBlendOptions>;
}
export interface IDataState {
    dataSourceNeedUpdate: boolean;
    dataMappingNeedUpdate: boolean;
    filterNeedUpdate: boolean;
    featureScaleNeedUpdate: boolean;
    StyleAttrNeedUpdate: boolean;
}
export interface ILayerModelInitializationOptions {
    moduleName: string;
    vertexShader: string;
    fragmentShader: string;
    triangulation: Triangulation;
}
export interface ILayerModel {
    render(): void;
    getUninforms(): IModelUniform;
    getDefaultStyle(): unknown;
    getAnimateUniforms(): IModelUniform;
    buildModels(): IModel[];
    needUpdate(): boolean;
}
export interface IModelUniform {
    [key: string]: IUniform;
}
export interface IPickedFeature {
    x: number;
    y: number;
    lnglat?: {
        lng: number;
        lat: number;
    };
    feature?: unknown;
}
export interface IActiveOption {
    color: string | number[];
}
export interface ILayer {
    id: string;
    type: string;
    name: string;
    inited: boolean;
    zIndex: number;
    plugins: ILayerPlugin[];
    layerModelNeedUpdate: boolean;
    layerModel: ILayerModel;
    dataState: IDataState;
    pickedFeatureID: number;
    hooks: {
        init: SyncBailHook<void, boolean | void>;
        afterInit: SyncBailHook<void, boolean | void>;
        beforeRenderData: SyncWaterfallHook<boolean | void>;
        beforeRender: SyncBailHook<void, boolean | void>;
        afterRender: SyncHook<void>;
        beforePickingEncode: SyncHook<void>;
        afterPickingEncode: SyncHook<void>;
        beforeHighlight: SyncHook<[number[]]>;
        beforeSelect: SyncHook<[number[]]>;
        afterSelect: SyncHook<void>;
        afterHighlight: SyncHook<void>;
        beforeDestroy: SyncHook<void>;
        afterDestroy: SyncHook<void>;
    };
    models: IModel[];
    sourceOption: {
        data: any;
        options?: ISourceCFG;
    };
    multiPassRenderer: IMultiPassRenderer;
    getLayerConfig(): Partial<ILayerConfig & ISceneConfig>;
    getContainer(): Container;
    setContainer(container: Container): void;
    setCurrentPickId(id: number | null): void;
    getCurrentPickId(): number | null;
    buildLayerModel(options: ILayerModelInitializationOptions & Partial<IModelInitializationOptions>): IModel;
    init(): ILayer;
    scale(field: string | IScaleOptions, cfg: IScale): ILayer;
    size(field: StyleAttrField, value?: StyleAttributeOption): ILayer;
    color(field: StyleAttrField, value?: StyleAttributeOption): ILayer;
    shape(field: StyleAttrField, value?: StyleAttributeOption): ILayer;
    label(field: StyleAttrField, value?: StyleAttributeOption): ILayer;
    animate(option: Partial<IAnimateOption> | boolean): ILayer;
    filter(field: string, value: StyleAttributeOption): ILayer;
    active(option: IActiveOption | boolean): ILayer;
    setActive(id: number | {
        x: number;
        y: number;
    }, option?: IActiveOption): void;
    select(option: IActiveOption | false): ILayer;
    setSelect(id: number | {
        x: number;
        y: number;
    }, option?: IActiveOption): void;
    style(options: unknown): ILayer;
    hide(): ILayer;
    show(): ILayer;
    setIndex(index: number): ILayer;
    isVisible(): boolean;
    setMaxZoom(min: number): ILayer;
    setMinZoom(max: number): ILayer;
    setBlend(type: keyof typeof BlendType): void;
    render(): ILayer;
    destroy(): void;
    source(data: any, option?: ISourceCFG): ILayer;
    setData(data: any, option?: ISourceCFG): ILayer;
    fitBounds(): ILayer;
    /**
     * 向当前图层注册插件
     * @param plugin 插件实例
     */
    addPlugin(plugin: ILayerPlugin): ILayer;
    getSource(): ISource;
    setSource(source: ISource): void;
    setEncodedData(encodedData: IEncodeFeature[]): void;
    getEncodedData(): IEncodeFeature[];
    getScaleOptions(): IScaleOptions;
    /**
     * 事件
     */
    on(type: string, hander: (...args: any[]) => void): void;
    off(type: string, hander: (...args: any[]) => void): void;
    emit(type: string, hander: unknown): void;
    once(type: string, hander: (...args: any[]) => void): void;
    /**
     * JSON Schema 用于校验配置项
     */
    getConfigSchemaForValidation(): object;
    isDirty(): boolean;
    /**
     * 直接调用拾取方法，在非鼠标交互场景中使用
     */
    pick(query: {
        x: number;
        y: number;
    }): void;
    updateLayerConfig(configToUpdate: Partial<ILayerConfig | unknown>): void;
    setAnimateStartTime(): void;
    getLayerAnimateTime(): number;
}
/**
 * Layer 插件
 */
export interface ILayerPlugin {
    apply(layer: ILayer, services: {
        rendererService: IRendererService;
        mapService: IMapService;
        styleAttributeService: IStyleAttributeService;
        postProcessingPassFactory: (name: string) => IPostProcessingPass<unknown>;
        normalPassFactory: (name: string) => IPass<unknown>;
    }): void;
}
/**
 * Layer 初始化参数
 */
export interface ILayerConfig {
    colors: string[];
    size: number;
    shape: string;
    shape2d: string[];
    shape3d: string[];
    scales: {
        [key: string]: IScale;
    };
    minZoom: number;
    maxZoom: number;
    visible: boolean;
    zIndex: number;
    autoFit: boolean;
    name: string;
    blend: keyof typeof BlendType;
    pickedFeatureID: number;
    enableMultiPassRenderer: boolean;
    passes: Array<string | [string, {
        [key: string]: unknown;
    }]>;
    /**
     * 开启拾取
     */
    enablePicking: boolean;
    /**
     * 开启高亮
     */
    enableHighlight: boolean;
    enableSelect: boolean;
    /**
     * 高亮颜色
     */
    highlightColor: string | number[];
    selectColor: string | number[];
    active: boolean;
    activeColor: string | number[];
    /**
     * 开启 TAA
     */
    enableTAA: boolean;
    /**
     * 相机抖动程度
     */
    jitterScale: number;
    /**
     * 开启光照
     */
    enableLighting: boolean;
    animateOption: Partial<IAnimateOption>;
    onHover(pickedFeature: IPickedFeature): void;
    onClick(pickedFeature: IPickedFeature): void;
}
/**
 * 提供 Layer 管理服务
 */
export interface ILayerService {
    clock: Clock;
    add(layer: ILayer): void;
    initLayers(): void;
    startAnimate(): void;
    stopAnimate(): void;
    getLayers(): ILayer[];
    getLayer(name: string): ILayer | undefined;
    remove(layer: ILayer): void;
    updateRenderOrder(): void;
    renderLayers(): void;
    destroy(): void;
}
